# Adapted from a question by Carl Boettiger.

library(XML)

xml = '<tree xmlns:r="http://www.r-project.org" xmlns:xsi="http://www.xml.org" id="tree2" xsi:type="nex:IntTree" label="tree2" r:type="list">
<node id="tree2n1" label="n1"/>
<node id="tree2n2" label="n2" otu="t1"/>
<node id="tree2n3" label="n3"/>
<node id="tree2n4" about="#tree2n4" label="n4">
<meta
id="tree2dict1"
property="cdao:has_tag"
content="true"
xsi:type="nex:LiteralMeta"
datatype="xsd:boolean"/>
</node>
<node id="tree2n5" label="n5" otu="t3"/>
<node id="tree2n6" label="n6" otu="t2"/>
<node id="tree2n7" label="n7"/>
<node id="tree2n8" label="n8" otu="t5"/>
<node id="tree2n9" label="n9" otu="t4"/>
<edge source="tree2n1" target="tree2n3" id="tree2e1" length="1"/>	
<edge source="tree2n1" target="tree2n2" id="tree2e2" length="2"/>
<edge source="tree2n3" target="tree2n4" id="tree2e3" length="3"/>
<edge source="tree2n3" target="tree2n7" id="tree2e4" length="1"/>
<edge source="tree2n4" target="tree2n5" id="tree2e5" length="2"/>
<edge source="tree2n4" target="tree2n6" id="tree2e6" length="1"/>
<edge source="tree2n7" target="tree2n8" id="tree2e7" length="1"/>
<edge source="tree2n7" target="tree2n9" id="tree2e8" length="1"/>
</tree>'


setClass("meta", representation(id = "character", property = "character", content = "logical", type = "character", datatype = "character"))

setClass("node",
    representation(id    = "character",
                   label = "character",
                   otu   = "character",
                   about = "character",
                   meta = "meta"))
setClass("edge",
    representation(source = "character",
                   target = "character",
                   id     = "character",
                   length = "numeric"))


# Our tree has lists of nodes and edges.
# So we define lists that validate their elements

setClass("ListOfnode", contains = "list",
          validity = function(object)
                       if(!all(sapply(object, is, "node")))
                          "not all elements are node objects"
                       else
                         TRUE)

setClass("ListOfedge", contains = "list",
          validity = function(object)
                       if(!all(sapply(object, is, "edge")))
                          "not all elements are edge objects"
                       else
                         TRUE)

# We could have done this with
setClass("ListOf",
          representation(elType = "character"),  contains = "list",
          validity = function(object)
                       if(!all(sapply(object, is, object@elType)))
                          "not all elements are edge objects"
                       else
                         TRUE)
setClass("ListOfedge", contains = "ListOf", prototype = prototype(elType = "edge"))
setClass("ListOfnode", contains = "ListOf", prototype = prototype(elType = "node"))


setClass("tree",
         representation(id         = "character", 
                        'xsi:type' = "character", 
                        label      = "character", 
                        node       = "ListOfnode", 
                        edge       = "ListOfedge",
                        type = "character"))  # type is test matching of r:type to type


setAs("XMLInternalElementNode", "node", function(from) xmlToS4(from))
setAs("XMLInternalElementNode", "meta", function(from) xmlToS4(from))
setAs("XMLInternalElementNode", "edge", function(from) xmlToS4(from))

setAs("XMLInternalElementNode", "tree",
       function(from) {
         obj = new("tree")
         kids = xmlChildren(from)
         obj@edge = new("ListOfedge", lapply(kids[names(kids) == "edge"], as, "edge"))
         obj@node = new("ListOfnode", lapply(kids[names(kids) == "node"], as, "edge"))
         ats = xmlAttrs(from, addNamespacePrefix = TRUE)
         obj = addXMLAttributes(obj, ats)
         obj
       })



root = xmlRoot(xmlParse(xml))
v = as(root, "tree")


tr = xmlToS4(xmlRoot(xmlParse(xml)))
